;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module Entity

.include "man/entity.inc"
.include "globals.inc"

.area _DATA

enemy_array::       
 .ds ENEMY_TOTAL_SIZE
 .db 0
;
player::
 .ds PLAYER_SIZE
;
shot_enemy_array:
 .ds SHOT_ENEMY_TOTAL_SIZE
 .db 0
;
shot_player_array:
 .ds SHOT_PLAYER_TOTAL_SIZE
 .db 0
;
entity_array_list:
    .db SHOT_ENEMY_TOTAL_SIZE
    .dw shot_enemy_array
    .db ENEMY_TOTAL_SIZE
    .dw enemy_array
    .db SHOT_PLAYER_TOTAL_SIZE
    .dw shot_player_array
    .db 0
;
next_free_entity_array:
    .dw shot_enemy_array
    .db 0
    .db SHOT_SIZE
    .dw enemy_array
    .db 0
    .db ENEMY_SIZE
    .dw shot_player_array
    .db 0
    .db SHOT_SIZE
;
tamaños_max:
    .db SHOT_ENEMY_MAX
    .db ENEMY_MAX
    .db SHOT_PLAYER_MAX
;
tamaños_uno:
    .db SHOT_SIZE
    .db ENEMY_SIZE
    .db SHOT_SIZE
;

.area _CODE

;Input  - A: Entity type
;Output - BC: Entity size
;Output - DE: Entity array
;Destroys AF,BC,DE,HL
man_entity_arrayAndSizeFromType:
    dec a
    ld b,#0
    ld c,a
    ld hl,#tamaños_uno
    add hl,bc   ;(HL) = Size
    ld e,(hl)   ;E = Size

    add a
    add c
    ld c,a  ;BC = (Type-1)*3
    ld hl,#entity_array_list
    add hl,bc   ;(HL) = entity array
    ld c,e  ;BC = Entity size
    inc hl
    ld e,(hl)
    inc hl
    ld d,(hl)   ;DE = entity array
    ret
;
man_entity_arrayListInit:
    ld a,(hl)
    or a
    ret z

    inc hl
    ld e,(hl)
    inc hl
    ld d,(hl)   ;DE holds memory location
    ld (ix),e
    ld 1(ix),d
    ld 2(ix),#0
    ld bc,#4
    add ix,bc

    ld c,a      ;BC holds size


    xor a       ;A holds 0
    inc hl
    push hl
    call cpct_memset_asm
    pop hl
    jr man_entity_arrayListInit
;
;Configuración inicial
man_entity_init::
    ld hl, #entity_array_list
    ld ix, #next_free_entity_array
    call man_entity_arrayListInit

    ld bc,#PLAYER_SIZE
    xor a
    ld de,#player
    call cpct_memset_asm


    
    ;ld hl,#num_entities        .db 0 Funciona?
    ;ld (hl),#0
    ;ld hl,#final
    ;ld (hl),#0
    ret
;

;Aumenta el contador de entidades en tamaño de entidad
;Input  - A:  Entity type
;Output - HL: Puntero a nueva entidad
;Output - DE: Puntero a tamaño de entidad

;Destruye AF,BC,DE,HL
;Antes de llamarse se debe comprobar que hay hueco libre con man_entity_freeSpace
man_entity_create::
    and #0x0F   ;Toma solo los primeros 4 bits del tipo
    ;Multiplicar por 4 el tipo
     ld  d,#0
     dec a
     add a
     add a
     ld  e,a ;DE = (A-1)*4
    ;
    ;Obtener puntero a next_free_entity según el tipo
     ld  hl,#next_free_entity_array
     add hl,de   ;HL = next_free_entity pointer
     ld e,l
     ld d,h      ;DE = next_free_entity_pointer
    ;
    ;Obtener nueva dirección de siguiente entidad
     ld  c,(hl)
     inc hl
     ld  b,(hl)  ;BC = next_free_entity
     inc hl
     inc (hl)    ;num entities += 1
     inc hl
     ld l,(hl)
     ld h,#0     ;HL = Tamaño entidad
     add hl,bc   ;HL = next_free_entity + Tamaño
    ;
    ;Crear nueva entidad
     ld a, #TYPE_DEFAULT
     ld (bc), a
    ;Actualizar next_free_entity
     ex de,hl    ;DE = next_free_entity + Tamaño
                 ;HL = next_free_entity_pointer
     ld (hl),e
     inc hl
     ld (hl),d   ;next_free_entity actualizado
     inc hl
     inc hl
    ;
    ;Cargar puntero a nueva entidad en HL
     ex de,hl    ;DE = puntero a tamaño
     ld l,c
     ld h,b      ;HL = nueva entidad
    ;
    ret
;

;Input - DE dirección de entidad a clonar
;Devuelve en HL la derección de la entidad clonada
;Destruye AF,BC,HL
man_entity_clone::
    push de
    ld a,(de)
    call man_entity_create
    ld  a,(de)              ;A  = tamaño entidad
    pop de
    push de                 ;DE = vieja entidad
    push hl                 ;HL = nueva entidad
    ex de,hl
    ld b,#0
    ld c,a
    call cpct_memcpy_asm
    pop hl
    pop de
    ret
;

;Input  - DE = dirección de entidad a destruir
man_entity_destroy::
    ld a,(de)
    and a,#TYPE_DEFAULT ;A = entity type
    dec a
    add a
    add a   ;A = (A-1)*4

    ld b,#0
    ld c,a
    ld hl, #next_free_entity_array
    add hl,bc   ;HL = next_free_entity pointer

    inc hl
    inc hl
    dec (hl)    ;Redice el contador de entidades
    inc hl
    ld  a,(hl)  ;A = entity tam
    dec hl
    dec hl
    dec hl      ;HL = next_free_entity pointer

    
    push de
    push hl
    ld e,(hl)
    inc hl
    ld d,(hl)
    ex de,hl    ;HL = next_free_entity

    neg
    ld  b,#0xFF
    ld  c,a
    add hl,bc
    ex de,hl    ;DE = new next_free_entity
    pop hl      ;HL = next_free_entity pointer
    ld  (hl),e
    inc hl
    ld  (hl),d  ;next_free_entity_pointer actualizado

    pop hl
    ex de,hl
    push hl
    neg
    ld  b,#0
    ld  c,a
    call cpct_memcpy_asm         ;Mueve la última al hueco a destruir
    pop hl
    ld  (hl),#TYPE_INVALID       ;Invalida la última posición

    ret
;

;IX = dirección de entidad a marcar
man_entity_set4Destruction::
    push af
    ld a,(ix)
    or #TYPE_DEAD
    ld (ix),a
    pop af
    ret
;

;Input  - DE:   Entidad destino
;Input  - C:    diff X
;Input  - B:    diff Y
;Input  - IX:   Entidad origen
;Destroys AF, IY
man_entity_setRelativePos::
    push de
    pop iy

    ld   a, entity_x(ix)
    add  c
    ld   entity_x(iy),a

    ld   a, entity_y(ix)
    add  b
    ld   entity_y(iy),a
    ret
;

man_entity_update::
    ld hl,#entity_array_list
    list_scan_loop:
     ld a,(hl)
     or a
     ret z   ;Fin de la lista
     inc hl
     ld e,(hl)
     inc hl
     ld d,(hl)   ;DE = entity_array
     inc hl      ;HL = next array
     push hl     ;Guarda en el stack el siguiente array
     ld a,(de)
     and #0x0F
     ld b,#0
     ld c,a
     dec c
     ld hl,#tamaños_uno
     add hl,bc
     ld c,(hl)
     push bc     ;Guarda en el stack el tamaño del array

     update_loop:
      ;Test ending
       ld a,(de)
       cp #TYPE_INVALID
       jr z, end_update_loop
      ;Test type
       ld a,(de)
       and #TYPE_DEAD
       cp #TYPE_DEAD 
       jr nz, increase_index
      ;Destroy
       push de
       call man_entity_destroy
       pop de
       jr update_loop
      increase_index:
       pop hl
       push hl
       add hl,de
       ex de,hl   ;Actualiza puntero
      jr update_loop
      
     end_update_loop:
      pop bc
      pop hl
      jr list_scan_loop
;


;Input - HL: function's address
;Calls the function in HL passing the entity's pointer on IX
;for all entities not invalid
;Destroys AF,BC,DE
man_entity_forAll::

    push hl
    ld bc,#forPlayer_return
    push bc
    ld ix,#player
    jp (hl)
    forPlayer_return:
    pop hl

    ld de,#entity_array_list

    forAll_list_scan_loop:
    ex de,hl
    ld a,(hl)
    or a
    ret z

    inc hl
    ld c,(hl)
    inc hl
    ld b,(hl)
    inc hl
    push bc     ;BC = entity array -> IX
    pop ix      ;IX = entity_array
    push hl     ;Next array on stack
    ld a,entity_type(ix)
    dec a
    and #0x0F
    ld c, a
    ld b,#0
    ld hl,#tamaños_uno
    add hl,bc
    ld c,(hl)
    push bc     ;Put entity size on stack
    ex de,hl    ;HL = función

    e_forAll_loop:
     ;Comprueba si ha llegado al final
      ld a,#TYPE_INVALID
      cp entity_type(ix)
      jr z, all_done
     ;
     ld bc,#e_for_all_return ;Punto de retorno de la función
     push hl
     push bc
     jp (hl)

     e_for_all_return:
      pop hl    ;Recupera la función
     ;Aumenta el puntero de IX en el tamaño de la entidad
      pop de
      push de
      add ix,de
    jr e_forAll_loop

    all_done:
     pop bc ;Remove size from stack
     pop de
     jr forAll_list_scan_loop
;

;Input - HL: function's address
;Input - A:  entity type
;Calls the function in HL passing the entity's pointer on IX
;for all entities which's type matches the signature
;Destroys AF,BC,DE
man_entity_forAll_matching::
    push hl
    and #0x0F

    call man_entity_arrayAndSizeFromType
    pop hl
    push de
    pop ix  ;IX = entity array
    push bc ;Entity size on stack

    e_forAll_loop_matching:
     ;Comprueba si ha llegado al final
      ld a,#TYPE_INVALID
      cp entity_type(ix)
      jr z, all_done_matching
      
     ;Calls the function
      push hl
      ld bc,#e_for_all_return_point_matching ;Punto de retorno de la función
      push bc
      jp (hl)
     e_for_all_return_point_matching:
     pop hl
     ;Aumenta el puntero de IX en el tamaño de la entidad
     update_pointer_matching:
      pop de
      push de
      add ix,de
    jr e_forAll_loop_matching

    all_done_matching:
    pop de
    ret
;

;Input - HL: function's address
;Input - A: bit signature
;Calls the function in HL passing the first entity's pointer on IX
;and the second entity's pointer in DE
;for all entity pairs which's type matches the signature
;Destroys BC,DE
;man_entity_forAllPairs_matching::
;    ld ix,#entities
;    push af
;    e_left_entity_loop:
;     ;Comprueba si ha llegado al final
;       ld a,#TYPE_INVALID
;       cp type(ix)
;       jr z, all_done_pairs_matching
;
;      ;Checks if entity meets condition
;       pop bc        ;Bit mask on b
;       ld a,b        ;Bit mask on A
;       and cmps(ix)  ;A = mask & e_type
;       cp b          ;Z = 1 if (type & mask) == mask
;       push bc
;       jr nz, update_pointer_left_entity
;      ;Loads next entity on DE
;       push ix
;       pop de
;       ex de,hl      ;HL = left entity; DE = function; IX = left entity
;       ld bc,#TAM
;       add hl,bc
;       ex de,hl      ;HL = function; DE = right entity; IX = left entity
;
;      e_right_entity_loop:
;       ;Comprueba si ha llegado al final
;        ld a,#TYPE_INVALID
;        ex de,hl
;        cp (hl)
;        ex de,hl
;        jr z, update_pointer_left_entity
;       ;Checks if entity meets condition
;        pop bc        ;Bit mask on b
;        ld a,b        ;Bit mask on A
;        and cmps(ix)  ;A = mask & e_type
;        cp b          ;Z = 1 if (type & mask) == mask
;        push bc
;        jr nz, update_pointer_right_entity
;
;         ;Calls the function
;          push hl
;          ld bc,#e_for_all_pairs_matching_return_point ;Punto de retorno de la función
;          push bc
;          jp (hl)
;        
;         e_for_all_pairs_matching_return_point:
;         pop hl
;
;
;       ;Aumenta el puntero de DE en el tamaño de la entidad
;       update_pointer_right_entity:
;        ex de,hl
;        ld bc,#TAM
;        add hl,bc
;        ex de,hl
;      jr e_right_entity_loop
;     ;Aumenta el puntero de IX en el tamaño de la entidad
;     update_pointer_left_entity:
;      ld de,#TAM
;      add ix,de
;    jr e_left_entity_loop
;
;    all_done_pairs_matching:
;    pop af
;    ret
;;
;
;
;Input  - A: Entity type
;Devuelve en a el resultado de Cantidad-num_entities
;Destruye AF,BC,HL
man_entity_freeSpace::
    dec a
    ld c,a
    ld b,#0
    ld hl,#tamaños_max
    add hl,bc
    ld a,(hl)
    neg
    ld h,b
    ld l,c
    add hl,hl
    add hl,hl
    inc hl
    inc hl
    ld  bc, #next_free_entity_array
    add hl, bc
    add (hl)
    neg
    ret 
;

man_entity_no_enemies::
    ld hl,#next_free_entity_array
    ld a,#TYPE_ENEMY
    dec a
    add a
    add a
    inc a
    inc a
    ld c,a
    xor b
    add hl,bc
    xor a
    cp (hl)
    ret
;