;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module Game

.include "cpctelera.h.s"
.include "globals.inc"
.include "man/entity.inc"
.include "sys/render.inc"
.include "man/entity_templates.inc"
.include "man/game.inc"

FINAL_LEVEL = 4
.area _DATA

enemy_spawn_counter:
    .db 0
;

game_state:
    .db 1
;
squid_boss_shot_iterator:
    .db 16
;


.area _CODE

score::
  .asciz '000000'
uscor::
    ld a,(hl)           ; current value of digit.
    add a,b             ; add points to this digit.
    ld (hl),a           ; place new digit back in string.
    cp #58               ; more than ASCII value '9'?
    ret c               ; no - relax.
    sub #10              ; subtract 10.
    ld (hl),a           ; put new character back in string.
uscor0:
    dec hl              ; previous character in string.
    inc (hl)            ; up this by one.
    ld a,(hl)           ; what's the new value?
    cp #58               ; gone past ASCII nine?
    ret c               ; no, scoring done.
    sub #10              ; down by ten.
    ld (hl),a           ; put it back
    jp uscor0           ; go round again.

;Input  - A: Game state
man_game_setGameState::
    ld (game_state),a
    ret
;
man_game_getGameState::
    ld a,(game_state)
    ret
;

;; En BC está el template a crear
;; En A está el tipo del template a crear
;; Devuelve en DE la entidad
man_game_create_template_entity::
    push bc
    push af
    call man_entity_freeSpace
    or   a
    jr   z, no_more_space

    pop  af
    call man_entity_create  ;; HL -> puntero a nueva entidad, DE -> puntero a tamaño entidad
    ld   a, (de)
    pop  bc                 ;; BC -> puntero a entidad origen
    ex   de, hl             ;; HL -> puntero a tamaño entidad, DE -> puntero a nueva entidad
    ld   h, b
    ld   l, c               ;; HL -> puntero a entidad origen
    ld   b, #0
    ld   c, a               ;; BC -> tamaño entidad
    push de                 ;; TODO -> checkear si es necesario
    call cpct_memcpy_asm
    pop  de
    jr end_freeSpace

    no_more_space:
        pop af
        pop bc

    end_freeSpace:
    ret
;

man_game_player_shot::
    ld   bc, #shot_player_template
    ld   a, #TYPE_PLAYER_SHOT
    call man_game_create_template_entity
    ex   de, hl

    ;; Position X
     push hl
     ld   bc, #entity_x
     add  hl, bc
     ld   a, entity_x(ix)
     add  #4       ;; BORRAR AL CAMBIAR EL SPRITE
     ld   (hl), a
     pop  hl
    ;; Position Y
     push hl
     ld   bc, #entity_y
     add  hl, bc
     ld   a, entity_y(ix)
     add  a, #10    ;; BORRAR AL CAMBIAR EL SPRITE
     ld   (hl), a
     pop  hl
    ;; Velocity

    ret
;

man_game_enemy_create_floating_enemy::
    ld   a, #TYPE_ENEMY
    ld   bc, #boss_squid_tentacle_template
    call man_game_create_template_entity

    ex   de, hl
    ld   bc, #entity_x
    add  hl, bc
    ld   a, entity_x(ix)
    add  a,#-10
    ld   (hl), a
    ld   bc, #entity_y - #entity_x
    add  hl, bc
    ld   a, entity_y(ix)
    ld   (hl), a

    ret
;

man_game_enemy_shot::
    ;; Create enemy shot
    ld   bc, #shot_enemy_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ret
;

man_game_enemy_vertical_shot::
    ;; Create enemy shot
    ld   bc, #shot_enemy_vertical_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ex   de, hl

    push hl
    ld   bc, #entity_y
    add  hl, bc
    ld   a, entity_y(ix)
    add  #-1
    ld   (hl), a
    pop  hl

    push hl
    ld   bc, #entity_x
    add  hl, bc
    ld   a, entity_x(ix)
    add  #4
    ld   (hl), a
    pop  hl

    ret
;

man_game_enemy_4_shot::
    ld   a, (squid_boss_shot_iterator)
    start_iterator_squid_shot:
    or   a
    jr   z, end_iterator_squid_shot

    ;; Create enemy shot
    ld   bc, #shot_enemy_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ex   de, hl

    push hl
    ld   bc, #entity_x
    add  hl, bc
    ld   a, entity_x(ix)
    add  #-2
    ld   (hl), a
    pop  hl

    push hl
    ld   bc, #entity_y
    add  hl, bc
    ld   a, (squid_boss_shot_iterator)
    ld   b, a
    ld   a, entity_y(ix)
    add  b
    ld   (hl), a
    pop  hl

    ld   a, (squid_boss_shot_iterator)
    sub  a, #4
    ld   (squid_boss_shot_iterator), a
    jp   start_iterator_squid_shot

    end_iterator_squid_shot:
    ld   a, #16
    ld   (squid_boss_shot_iterator), a
    ret
;

man_game_init_all:
    call sys_ai_entities_init
    call sys_ai_movements_init
    call man_level_setup
    call man_game_score_init
    call sys_render_init
    call man_entity_init
    ld a,#STATE_PLAY
    call man_game_setGameState
    call sys_ai_movements_init
    call sys_ai_entities_init
    call man_game_player_init
    ret
;

man_game_init::
    ld a,#1
    ld hl,#current_level
    ld (hl),a
    call man_game_init_all

    ;; MotherShip
    ;;ld   bc, #mothership_template
    ;;call man_game_create_template_entity

    ;; ScoreBoard
    ;;ld a, #INIT_PLAYERSHIP_POS
;;
;;loop_template:
    ;;ld   bc, #playership_template
    ;;push af
    ;;call man_game_create_template_entity
    ;;pop af
    ;;sub  #10
    ;;ex   de, hl                             
    ;;ld   bc, #entity_x
    ;;add  hl, bc
    ;;ld   (hl), a
    ;;or   a
    ;;jr   nz, loop_template

    ;ld player_health(ix), #PlayerShip_0_INIT_HEALTH

    ret
;

man_game_play::
    loop:

;;    ld a, (enemy_spawn_counter)
;;    inc a
;;    ld (enemy_spawn_counter), a
;;
;;    and #63
;;    jr nz, no_spawn_enemy
;;

    no_spawn_enemy:
    call sys_ai_update
    call sys_physics_update
    call sys_animations_update
    call cpct_waitVSYNC_asm
    call sys_collisions_update
    call sys_render_update
    call man_entity_update

    call man_level_isOver
    jr nz, level_finished
    call man_level_update
    jp check_game_over

     level_finished:

     call man_entity_no_enemies
     jp nz, check_game_over
     ld hl,#current_level
     inc (hl)
     call man_level_setup
     call update_score
     ld a,(current_level)
     cp #FINAL_LEVEL
     jp z, end_card

    check_game_over:
    ld a, (game_state)
    or a
    jr nz, loop
    call man_menu_gameOver
    ld a,#1
    ld (game_state),a
    ret nz
    ld hl,#current_level
    call man_game_init_all
    jr loop
    end_card:
    call man_menu_endCard
    ret
;

;;man_game_create_enemy::
;;
;;    ld   hl, #m_lane_status
;;    inc  hl
;;    inc  hl
;;    ld   a, (hl)
;;    cp   #0
;;    jr   nz, not_create
;;
;;    call man_game_create_template_entity
;;    ex   de, hl
;;    ld   bc, #entity_x
;;    add  hl, bc
;;    ld   a, entity_x(ix)
;;    add  a,#4               ;TODO CONSTANT FOR dx
;;    ld   (hl), a
;;    ld   bc, #entity_vx - #entity_x
;;    add  hl, bc
;;    ld   a, entity_vx(ix)
;;    ld   (hl), a
;;
;;    ld   hl, #m_lane_status
;;    inc  hl
;;    inc  hl
;;    inc  (hl)
;;    not_create:
;;    ret
;

;;man_game_enemy_lane_down::
;;    ld  bc, #2        ;;Default line is 2
;;
;;    ld  h, entity_y(ix)
;;    ld  a, #LANE1_Y
;;    cp  h
;;    jp  c, man_game_enemy_lane_down_return
;;
;;    ld  a, #LANE2_Y
;;    cp  h
;;    jp  nc, check_down_lane
;;
;;    ld  bc, #1
;;
;;    check_down_lane:
;;    ;; check
;;    ld  a, entity_y(ix)
;;    ld  hl, #m_lane_status
;;    add hl, bc
;;    dec hl
;;    ld  a, (hl)
;;    or  a
;;    jr  nz, man_game_enemy_lane_down_return
;;
;;    ;; Create phantom clone entity to erase trail
;;    push hl
;;    push ix
;;    pop hl
;;    ex de, hl
;;    call man_entity_clone
;;    ;;ld bc, #cmps
;;    ;;add hl, bc
;;    ;;ld (hl), #CMP_RENDER
;;    ;;ld bc, #-cmps
;;    ;;add hl, bc
;;    ex de, hl
;;    call man_entity_set4Destruction
;;    pop hl
;;
;;    ;; go down
;;    ld  a, entity_y(ix)
;;    add a, #LANE_DY
;;    ld  entity_y(ix), a
;;
;;    inc (hl)
;;    inc hl
;;    ld  (hl), #0
;;
;;    man_game_enemy_lane_down_return:
;;    ret
;

;;man_game_entity_destroy::
;;    ld   a, entity_type(ix)
;;    and  #TYPE_PLAYER_SHOT
;;    cp   #TYPE_PLAYER_SHOT
;;    jr   nz, not_shot
;;
;;    ld   a, #0
;;    ld   (m_player_shot), a
;;    jp   not_enemy_shot
;;
;;    not_shot:
;;    ld   a, entity_type(ix)
;;    and  #TYPE_ENEMY_SHOT
;;    cp   #TYPE_ENEMY_SHOT
;;    jr   nz, not_enemy_shot
;;
;;    ld   a, #0
;;    ld   (m_enemy_shot), a
;;
;;    not_enemy_shot:
;;    push ix
;;    pop  de
;;    call man_entity_set4Destruction
;;    ret
;

;;man_game_enemy_shot::
;;
;;    ;; Check if there is a shot already
;;    ld   a, (m_enemy_shot)
;;    or   a
;;    jr   nz, not_create_shot_enemy
;;
;;    ld   bc, #shot_template
;;    call man_game_create_template_entity
;;
;;    ex   de, hl
;;    push hl
;;    ;; La velocidad debe ser la opuesta a la del disparo del jugador
;;    ld   bc, #entity_vy
;;    add  hl, bc
;;    ld   a, #-Vshot_0_VY
;;    ld   (hl), a
;;    pop  hl
;;
;;    push hl
;;    ;; La pos_y debe ser la del enemigo
;;    ld   bc, #entity_y
;;    add  hl, bc
;;    ld   a, entity_y(ix)
;;    ld   (hl), a
;;    pop  hl
;;
;;    ld   bc, #entity_x
;;    add  hl, bc
;;    ld   a, entity_x(ix)
;;    add  a, #2
;;    ld   (hl), a
;;    ld   a, #1
;;    ld   (m_enemy_shot), a
;;
;;    not_create_shot_enemy:
;;    ret
;

;;man_game_enemy_dash::
;;    ld  h, entity_y(ix)
;;    ld  a, #LANE0_Y
;;    cp  h
;;    jp  nz, man_game_enemy_dash_return
;;    ld  a, (m_enemy_flying)
;;    or  a
;;    jr  nz, man_game_enemy_dash_return
;;
;;    ld  entity_vy(ix), #1
;;    ld  hl, #m_lane_status
;;    dec (hl)
;;    ld  hl, #m_enemy_flying
;;    inc (hl)
;;
;;    man_game_enemy_dash_return:
;;    ret
;
update_score:
    push af
    push bc
    push de
    push hl
    push ix
    push iy
    ld b,#1
    ld hl, #score+#2
    call uscor
    ld hl,#0x010f
    call cpct_setDrawCharM0_asm
    ld de,#CPCT_VMEM_START_ASM
    ld b,#192
    ld c,#50
    call cpct_getScreenPtr_asm
    ld iy,#score
    call cpct_drawStringM0_asm
    pop iy
    pop ix
    pop hl
    pop de
    pop bc
    pop af
    ret
;

man_game_score_init:
    xor a
    ld hl,#score
    score_init:
    cp (hl)
    ret z
    ld (hl),#48
    inc hl
    jr nz, score_init
;
man_game_player_init:
    ld de, #player
    ld hl, #player_template
    ld bc, #PLAYER_SIZE
    call cpct_memcpy_asm
    ret
;