;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.include "globals.inc"
.include "cpctelera.h.s"
.area _DATA

START       = 0
KEYBOARD    = 1
JOYSTICK    = 2
pointer: .db START

up_pressed:     .db 0
down_pressed:   .db 0
enter_pressed:  .db 0
yes_no_pointer: .db 0
left_pressed:   .db 0
right_pressed:  .db 0
start:      .asciz 'START'
keyboard:   .asciz 'Keyboard'
joystick:   .asciz 'Joystick'
victory:    .asciz 'YOU WIN!'
final_score:.asciz 'FINAL SCORE:'
game_over:  .asciz 'GAME OVER'
continue:   .asciz 'CONTINUE?'
yes_no:     .asciz 'YES       NO'

.area _CODE

;TODO   - Draw the menu screen
man_menu_screen::
    ld   de, #_menu_song
    call cpct_akp_musicInit_asm

   ld hl,#_palette_menu
   ld de,#16
   call cpct_setPalette_asm


    ld de,#0x8000               
    xor a                       
    ld bc,#0xC828               
    call cpct_drawSolidBox_asm  
    ld de,#0x8028               
    xor a                       
    ld bc,#0xC828               
    call cpct_drawSolidBox_asm  

    ld de,#0x8000
    ld bc,#0x1411
    call cpct_getScreenPtr_asm
    ex de,hl
    ld hl,#_spr_title
    ld bc,#0x242D
    call cpct_drawSprite_asm

    ;TODO Dibujar los sprites del menú aquí

    ld hl,#0x0009
    call cpct_setDrawCharM0_asm

    ld de,#0x8000
    ld bc,#0x641E
    call cpct_getScreenPtr_asm
    ld iy,#start
    call cpct_drawStringM0_asm

    ld de,#0x8000
    ld bc,#0x7417
    call cpct_getScreenPtr_asm
    ld iy,#keyboard
    call cpct_drawStringM0_asm

    ld de,#0x8000
    ld bc,#0x8417
    call cpct_getScreenPtr_asm
    ld iy,#joystick
    call cpct_drawStringM0_asm


   ld l,#0x20
   call cpct_setVideoMemoryPage_asm

   cpctm_clearScreen_asm #0x00
    
    ret
;

;TODO   - Keyboard/Joystick selection
;Pressing start returns
man_menu_handler::
    loop:
    call man_menu_eraseChars

    call man_menu_keyControl
    ret nz
    
    call man_menu_drawChars
    call cpct_waitVSYNC_asm

    
    ;call cpct_scanKeyboard_asm
    ;call cpct_isAnyKeyPressed_asm
    jp loop

    ret
;

man_menu_drawChars:
    ld hl,#0x0004
    call cpct_setDrawCharM0_asm

    ld de,#0x8000
    ld a,(pointer)
    cp #START
    jp nz, is_keyboard
    ld bc,#0x641A
    jp get_pointer
    is_keyboard:
    cp #KEYBOARD
    jp nz, is_joystick
    ld bc,#0x7413
    jp get_pointer
    is_joystick:
    ld bc,#0x8413
    get_pointer:
    call cpct_getScreenPtr_asm
    ld e,#62
    call cpct_drawCharM0_asm

    ld de,#0x8000
    call man_game_getInputOption
    cp #0
    jp nz, control_joystick
    ld bc,#0x7437
    jp get_selection
    control_joystick:
    ld bc,#0x8437
    get_selection:
    call cpct_getScreenPtr_asm
    ld e,#43
    call cpct_drawCharM0_asm

    ret
;
man_menu_eraseChars:
    ld de,#0x8000
    ld a,(pointer)
    cp #START
    jp nz, is_keyboard_erase
    ld bc,#0x641A
    jp get_pointer_erase
    is_keyboard_erase:
    cp #KEYBOARD
    jp nz, is_joystick_erase
    ld bc,#0x7413
    jp get_pointer_erase
    is_joystick_erase:
    ld bc,#0x8413
    get_pointer_erase:
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1004
    call cpct_drawSolidBox_asm

    ld de,#0x8000
    call man_game_getInputOption
    cp #0
    jp nz, control_joystick_erase
    ld bc,#0x7437
    jp get_selection_erase
    control_joystick_erase:
    ld bc,#0x8437
    get_selection_erase:
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1004
    call cpct_drawSolidBox_asm

    ret
;
man_menu_keyControl:

    call cpct_scanKeyboard_asm
    ld hl,#Key_Q
    call cpct_isKeyPressed_asm
    ld a, (up_pressed)
    ld b,a
    jp z, up_not_pressed
    ld a,#1
    ld (up_pressed),a
    cp b
    jp z, check_down
    call man_menu_decPointer
    jp check_down

    up_not_pressed:
    ld a,#0
    ld (up_pressed),a

    check_down:

    ld hl,#Key_A
    call cpct_isKeyPressed_asm
    ld a, (down_pressed)
    ld b,a
    jp z, down_not_pressed
    ld a,#1
    ld (down_pressed),a
    cp b
    jp z, check_enter
    call man_menu_incPointer
    jp check_enter

    down_not_pressed:
    ld a,#0
    ld (down_pressed),a

    check_enter:
    ld hl,#Key_Return
    call cpct_isKeyPressed_asm
    ld a, (enter_pressed)
    ld b,a
    jp z, enter_not_pressed
    ld a,#1
    ld (enter_pressed),a
    cp b
    ret z
    call man_menu_enterHandler
    ret

    enter_not_pressed:
    ld a,#0
    ld (enter_pressed),a
    ret
;
man_menu_incPointer:
    ld a,(pointer)
    cp #2
    ret z
    inc a
    ld (pointer), a
    ret
;
man_menu_decPointer:
    ld a,(pointer)
    cp #0
    ret z
    dec a
    ld (pointer), a
    ret
;
man_menu_enterHandler:
    ld a,(pointer)
    cp #KEYBOARD
    jp nz, set_joystick
    ld a,#0
    call man_game_setInputOption
    ret
    set_joystick:
    cp #JOYSTICK
    ret nz
    ld a,#1
    call man_game_setInputOption
    ret
;
man_menu_gameOver::

    ld   de, #_death_song
    call cpct_akp_musicInit_asm


    ld  hl, #man_entity_set4Destruction
    call man_entity_forAll
    call sys_render_update
    ;call man_entity_update

    ld hl,#_main_palette
    ld de,#16
    call cpct_setPalette_asm

    ld hl,#0x0009
    call cpct_setDrawCharM0_asm

    ld de,#0xC000
    ld bc,#0x2416
    call cpct_getScreenPtr_asm
    ld iy,#game_over
    call cpct_drawStringM0_asm

    ld de,#0xC000
    ld bc,#0x4416
    call cpct_getScreenPtr_asm
    ld iy,#continue
    call cpct_drawStringM0_asm

    ld de,#0xC000
    ld bc,#0x7410
    call cpct_getScreenPtr_asm
    ld iy,#yes_no
    call cpct_drawStringM0_asm
    ld hl,#0x0004
    call cpct_setDrawCharM0_asm

    stay:

    call man_menu_eraseGameOverChar
    call man_menu_gameOverKeyboard
    jr nz, exit_game_over
    call man_menu_drawGameOverChar
    call cpct_waitVSYNC_asm
    jr stay

    exit_game_over:

    ;TODO LIMPIAR TEXTO EN PANTALLA
    call man_menu_clean_game_over

    
    ld a,(yes_no_pointer)
    ld c,a
    xor a
    ld (yes_no_pointer),a
    ld a,c
    cp #0

    ret
;
man_menu_clean_game_over:
    ld de,#0xC000
    ld bc,#0x2416
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1024
    call cpct_drawSolidBox_asm

    ld de,#0xC000
    ld bc,#0x4416
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1024
    call cpct_drawSolidBox_asm

    ld de,#0xC000
    ld bc,#0x740B
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1035
    call cpct_drawSolidBox_asm

    ret
;
man_menu_drawGameOverChar:
    ld a,(yes_no_pointer)
    or a
    jr z, points_yes
    ld bc,#0x7434
    jr get_yes_no_pointer
    points_yes:
    ld bc,#0x740B
    get_yes_no_pointer:
    ld de,#0xC000
    call cpct_getScreenPtr_asm
    ld e,#62
    call cpct_drawCharM0_asm

    ret
;
man_menu_gameOverKeyboard:

    call cpct_scanKeyboard_asm
    ld hl,#Key_O
    call cpct_isKeyPressed_asm
    ld a, (left_pressed)
    ld b,a
    jp z, left_not_pressed
    ld a,#1
    ld (left_pressed),a
    cp b
    jp z, check_right
    call man_menu_changeYESNO
    jp check_right

    left_not_pressed:
    ld a,#0
    ld (left_pressed),a

    check_right:

    ld hl,#Key_P
    call cpct_isKeyPressed_asm
    ld a, (right_pressed)
    ld b,a
    jp z, right_not_pressed
    ld a,#1
    ld (right_pressed),a
    cp b
    jp z, check_enter_over
    call man_menu_changeYESNO
    jp check_enter_over

    right_not_pressed:
    ld a,#0
    ld (right_pressed),a

    check_enter_over:
    ld hl,#Key_Return
    call cpct_isKeyPressed_asm
    ld a, (enter_pressed)
    ld b,a
    jp z, enter_not_pressed_over
    ld a,#1
    ld (enter_pressed),a
    cp b
    ret

    enter_not_pressed_over:
    ld a,#0
    ld (enter_pressed),a
    ret
;
man_menu_changeYESNO:
    ld a,(yes_no_pointer)
    xor #1
    ld (yes_no_pointer),a
    ret
;
man_menu_eraseGameOverChar:
    ld a,(yes_no_pointer)
    or a
    jr z, points_yes_erase
    ld bc,#0x7434
    jr get_yes_no_pointer_erase
    points_yes_erase:
    ld bc,#0x740B
    get_yes_no_pointer_erase:
    ld de,#0xC000
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1004
    call cpct_drawSolidBox_asm
    ret
;
man_menu_endCard::

    ld   de, #_victory_song
    call cpct_akp_musicInit_asm

    ld  hl, #man_entity_set4Destruction
    call man_entity_forAll
    call sys_render_update

    ld hl,#0x0009
    call cpct_setDrawCharM0_asm

    ld de,#CPCT_VMEM_START_ASM
    ld bc,#0x641E
    call cpct_getScreenPtr_asm
    ld iy,#score
    call cpct_drawStringM0_asm

    ld de,#CPCT_VMEM_START_ASM
    ld bc,#0x441A
    call cpct_getScreenPtr_asm
    ld iy,#victory
    call cpct_drawStringM0_asm

    ld de,#CPCT_VMEM_START_ASM
    ld bc,#0x5412
    call cpct_getScreenPtr_asm
    ld iy,#final_score
    call cpct_drawStringM0_asm

    victory_loop:
    call cpct_scanKeyboard_asm
    call cpct_isAnyKeyPressed_asm
    jr z, victory_loop
    end_loop:
      call cpct_scanKeyboard_asm
      ld hl, #Key_Return
      call cpct_isKeyPressed_asm
   jr z, end_loop

    call man_menu_clean_endCard
    ret
;
man_menu_clean_endCard:
    ld de,#0xC000
    ld bc,#0x641E
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1018
    call cpct_drawSolidBox_asm

    ld de,#0xC000
    ld bc,#0x441A
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1020
    call cpct_drawSolidBox_asm

    ld de,#0xC000
    ld bc,#0x5412
    call cpct_getScreenPtr_asm
    ex de,hl
    xor a
    ld bc,#0x1030
    call cpct_drawSolidBox_asm
    ret
;