;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module AiEntities

.include "man/entity.inc"
.include "sys/render.inc"
.include "man/entity_templates.inc"
.include "globals.inc"

.area _DATA

enemy_stop_counter_squid:
    .db 0
;

enemy_stop_counter_crab:
    .db 0
;

enemy_stop_counter_kraken:
    .db 0
;

first_time_stop:
    .db 0
;

storage_vx:
    .db 0
;

storage_vy:
    .db 0
;

spawn_tentacle:
    .db 1
;

dash_state:
    .db 0
;

tentacle_state:
    .db 1
;

mechaKraken_phase:
    .db 1
;

.area _CODE

sys_ai_entities_init::
    xor a
    ld  (enemy_stop_counter_squid), a
    ld  (enemy_stop_counter_crab), a
    ld  (enemy_stop_counter_kraken), a
    ld  (first_time_stop), a
    ld  (storage_vx), a
    ld  (storage_vy), a
    ld  (dash_state), a
    ld  a, #1
    ld  (spawn_tentacle), a
    ld  (tentacle_state), a
    ld  (mechaKraken_phase), a
    ret
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                      ;;
;;                          SQUID BOSS                                  ;;
;;                                                                      ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Polymorphic function that determines the squid boss behaviour
;; depending on the type of enemy and its funtion pointers
;; INPUT: enemy entity recieved in IX
sys_ai_behaviour_boss_squid::
    ;; Checks if the stop timer is set
    ld   a, (enemy_stop_counter_squid)
    cp   #1
    jr   nc, enemy_stop

    ;; If the wait has ended, the velocities are recovered
    ;; from the variables, in order to continue with the
    ;; same movement
    ld   a, (first_time_stop)
    or   a
    jr   z, continue_no_stop

    ld   a, (storage_vx)
    ld   entity_vx(ix), a
    ld   a, (storage_vy)
    ld   entity_vy(ix), a
    ld   a, #0
    ld   (first_time_stop), a

    continue_no_stop:
    ;; Every 64 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #63
    jr   nz, sys_ai_behaviour_squid_no_shot

    ;; Sets the wait for x frames
    ld   a, #43
    ld   (enemy_stop_counter_squid), a


    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_squid_no_shot
    push bc
    jp (hl)

    sys_ai_behaviour_squid_no_shot:

    call sys_ai_behaviour_circle

    increase_counter:
    ;ld   a, enemy_counter(ix)
    ;inc  a
    ;ld   enemy_counter(ix), a

    ;; Check life for spawning tentacle
    ld  a, enemy_health(ix)
    and #3
    jr  nz, #boss_spawn_tentacle

    ld  a, (spawn_tentacle)
    or  a
    ret nz
    ld  a, #1
    ld  (spawn_tentacle), a
    ret

    boss_spawn_tentacle:
    ld  a, (spawn_tentacle)
    or  a
    ret z
    call man_game_enemy_create_floating_enemy
    ld  a, #0
    ld  (spawn_tentacle), a
    ret

    ;; If it is the first frame where it has to stop
    ;; both velocities will be stored in variables in
    ;; order to recover them when the wait ends
    enemy_stop:
    ld   a, (first_time_stop)
    or   a
    jr   nz, no_first_time_stop
    ld   a, #1
    ld   (first_time_stop), a

    ld   a, entity_vx(ix)
    ld   (storage_vx), a
    ld   a, entity_vy(ix)
    ld   (storage_vy), a

    ;; And stops the entity
    ld   entity_vy(ix), #0
    ld   entity_vx(ix), #0

    no_first_time_stop:
    ld   a, (enemy_stop_counter_squid)
    dec  a
    ld   (enemy_stop_counter_squid), a

    jp   increase_counter
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                      ;;
;;                          KR-4B BOSS                                   ;;
;;                                                                      ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Polymorphic function that determines the crab boss behaviour
;; depending on the type of enemy and its funtion pointers
;; INPUT: enemy entity recieved in IX
sys_ai_behaviour_boss_crab::
    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #15
    jr   nz, sys_ai_behaviour_crab_boss_no_shot

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_crab_boss_return
    push bc
    jp (hl)
    sys_ai_behaviour_crab_boss_return:

    sys_ai_behaviour_crab_boss_no_shot:
    ;;ld   a, enemy_counter(ix)
    ;;inc  a
    ;;ld   enemy_counter(ix), a

    ;; Checks if the stop timer is set
    ld   a, (enemy_stop_counter_crab)
    cp   #1
    jr   nc, crab_boss_stop

    ;; Get the player position and store it in a
    push bc
    ld   bc, #player
    inc  bc
    inc  bc
    ld   a, (bc)
    inc  a
    pop  bc

    ;; Checks its position in relation with player
    ld   b, a
    ld   a, entity_y(ix)
    cp   b
    jr   c, player_is_down
    jr   z, player_located

    ld   entity_vy(ix), #-1
    jp   boss_crab_end

    player_located:
    ld   entity_vy(ix), #0

    ;; Sets the wait for x frames
    ld   a, #32
    ld   (enemy_stop_counter_crab), a
    jp   boss_crab_end

    crab_boss_stop:
    ld   a, (enemy_stop_counter_crab)
    dec  a
    ld   (enemy_stop_counter_crab), a
    ret

    player_is_down:
    ld   entity_vy(ix), #1

    boss_crab_end:
    call sys_ai_behaviour_up_down
    ret
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                      ;;
;;                          MECHA-KRAKEN                                ;;
;;                                                                      ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; INPUT: enemy entity receivd in IX
sys_ai_behaviour_boss_mechaKraken_tentacle:
    call sys_ai_behaviour_regenerate

    ld   a, enemy_counter(ix)
    and  #63
    ret  nz

    ld   a, enemy_counter(ix)
    and  #127
    jr   nz, sys_ai_behaviour_move


    ld   entity_vy(ix), #0

    ld   a, (mechaKraken_phase)
    cp   #3
    jr   z, stay_closed

    ld   a, b
    add  a
    ret  nc
    ld   a, (tentacle_state)
    inc  a
    ld   (tentacle_state), a
    ret

    stay_closed:
    ld   a, #2
    ld   (tentacle_state), a
    ret

    sys_ai_behaviour_move:

    ld   a, (mechaKraken_phase)
    cp   #3
    ret  z

    ld   a, (tentacle_state)
    and  #1
    jr   nz, sys_ai_behaviour_tentacle_open

    ld   a, b
    ld   entity_vy(ix), a
    ret

    sys_ai_behaviour_tentacle_open:
    ld   a, (mechaKraken_phase)

    cp   #4
    jr   nc, open_tentacles

    cp   #2
    jr   c, open_tentacles
    ret  nz

    ld   a, #3
    ld   (mechaKraken_phase), a
    ;;jp   stay_closed
    ret

    open_tentacles:
    ld   a, c
    ld   entity_vy(ix), a
    ret
;

sys_ai_behaviour_boss_mechaKraken_head::
    ld   a, (enemy_stop_counter_kraken)
    cp   #1
    jr   c, not_in_phase_3

    cp   #2
    jr   c, begin_phase_4

    mechaKraken_repair:
    ld   enemy_health(ix), #64
    ld   a, (mechaKraken_phase)
    cp   #3
    ret  c
    ld   a, (enemy_stop_counter_kraken)
    dec  a
    ld   (enemy_stop_counter_kraken), a
    ret

    begin_phase_4:
    ld   a, #4
    ld   (mechaKraken_phase), a
    ld   entity_vy(ix), #1
    ld   enemy_shot_pattern(ix), #sys_ai_shot_pattern_boss_mechaKraken_shot_2

    jp   mechaKraken_repair

    not_in_phase_3:
    ld   a, (mechaKraken_phase)
    cp   #4
    jr   nc, mechaKraken_shot_order

    ld   a, enemy_health(ix)
    and  #31
    jr   nz, no_change_to_phase_2

    ld   a, #2
    ld   (mechaKraken_phase), a
    dec  enemy_health(ix)
    ld   entity_vy(ix), #0
    ret

    no_change_to_phase_2:

    ld  a, (mechaKraken_phase)
    cp  #2
    jr  nc, mechaKraken_phase_2

    mechaKraken_shot_order:
    ld   l, enemy_shot_pattern(ix)
    ld   h, 1 + enemy_shot_pattern(ix)
    ld   bc, #sys_ai_behaviour_mechaKraken_boss_return
    push bc
    jp   (hl)

    sys_ai_behaviour_mechaKraken_boss_return:
    call sys_ai_behaviour_up_down
    ret

    mechaKraken_phase_2:
    ld   a, #255
    ld   (enemy_stop_counter_kraken), a
    ret
;

sys_ai_behaviour_boss_mechaKraken_tentacle_bottom::
    ld   a, (mechaKraken_phase)
    cp   #4
    jr   nc, invert_movement_bottom

    ld   b, #-1
    ld   c, #1
    call sys_ai_behaviour_boss_mechaKraken_tentacle
    ret

    invert_movement_bottom:
    ld   b, #1
    ld   c, #-1
    call sys_ai_behaviour_boss_mechaKraken_tentacle
    ret
;

sys_ai_behaviour_boss_mechaKraken_tentacle_top::
    ld   a, (mechaKraken_phase)
    cp   #4
    jr   nc, invert_movement_top

    ld   b, #1
    ld   c, #-1
    call sys_ai_behaviour_boss_mechaKraken_tentacle
    ret

    invert_movement_top:
    ld   b, #-1
    ld   c, #1
    call sys_ai_behaviour_boss_mechaKraken_tentacle
    ret
;

sys_ai_destroy_one_enemy:
    ld hl, #man_entity_set4Destruction
    ld bc,#update_one_entity_return
    push bc
    jp (hl)
    update_one_entity_return:
    ret
;

sys_ai_destroy_all_enemies::
    ld hl, #sys_ai_destroy_one_enemy
    ld a, #TYPE_ENEMY
    call man_entity_forAll_matching
    ret
;
sys_ai_destroy_all_enemies_kraken::
    push ix
    ld hl, #sys_ai_destroy_one_enemy
    ld a, #TYPE_ENEMY
    call man_entity_forAll_matching

    pop ix

    ld bc, #enemy_explosion_template
    ld a,#TYPE_ENEMY
    call man_game_create_template_entity
    ex de,hl
    ld de, #entity_x
    add hl,de
    ld a,entity_x(ix)
    add #4
    ld (hl),a
    inc hl
    ld a,entity_y(ix)
    add #2
    ld (hl),a

    ld bc, #enemy_explosion_template
    ld a,#TYPE_ENEMY
    call man_game_create_template_entity
    ex de,hl
    ld de, #entity_x
    add hl,de
    ld a,entity_x(ix)
    add #2
    ld (hl),a
    inc hl
    ld a,entity_y(ix)
    add #8
    ld (hl),a

    ld bc, #enemy_explosion_template
    ld a,#TYPE_ENEMY
    call man_game_create_template_entity
    ex de,hl
    ld de, #entity_x
    add hl,de
    ld a,entity_x(ix)
    add #7
    ld (hl),a
    inc hl
    ld a,entity_y(ix)
    add #12
    ld (hl),a

    ret
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                      ;;
;;                          ENEMIES                                     ;;
;;                                                                      ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Polymorphic function that determines the enemy behaviour
;; depending on the type of enemy and its funtion pointers
;; INPUT: enemy entity recieved in IX
sys_ai_behaviour_enemy::
    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #31
    jr   nz, sys_ai_behaviour_enemy_no_shot

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_enemy_return
    push bc
    jp (hl)
    sys_ai_behaviour_enemy_return:

    ;; After 256 frames, the enemy dies
    sys_ai_behaviour_enemy_no_shot:
    ld   a, enemy_counter(ix)
    ;;inc  a
    ;;ld   enemy_counter(ix), a
    cp   #0xFF
    jr   z, sys_ai_behaviour_enemy_destroy

    call sys_ai_behaviour_up_down
    ret

    sys_ai_behaviour_enemy_destroy:
    call man_entity_set4Destruction
    ret
;

sys_ai_behaviour_enemy_arco_shot::
    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #63
    jr   nz, sys_ai_behaviour_enemy_no_shot_arco_shot

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_enemy_return_arco_shot
    push bc
    jp (hl)
    sys_ai_behaviour_enemy_return_arco_shot:

    ;; After 256 frames, the enemy dies
    sys_ai_behaviour_enemy_no_shot_arco_shot:
    ld   a, enemy_counter(ix)
    ;;inc  a
    ;;ld   enemy_counter(ix), a
    cp   #0xFF
    jr   z, sys_ai_behaviour_enemy_destroy_arco_shot

    call sys_ai_behaviour_up_down
    ret

    sys_ai_behaviour_enemy_destroy_arco_shot:
    call man_entity_set4Destruction
    ret
;

sys_ai_behaviour_floating_enemy::
    ld  enemy_health(ix), #31
    ld a,enemy_counter(ix)
    cp #0xFF
    ret nz
    call man_entity_set4Destruction
    ret
;
sys_ai_behaviour_still_enemy::
    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #63
    jr   nz, sys_ai_behaviour_enemy_still_return

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_enemy_still_return
    push bc
    jp (hl)
    sys_ai_behaviour_enemy_still_return:
    ld   a, enemy_counter(ix)
    ;;inc  a
    ;;ld   enemy_counter(ix), a
    cp   #0xFF
    ret nz
    call man_entity_set4Destruction
    ret
;

sys_ai_behaviour_horizontal_enemy::
    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #15
    jr   nz, sys_ai_behaviour_enemy_no_v_shot

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_h_enemy_return
    push bc
    jp (hl)
    sys_ai_behaviour_h_enemy_return:

    ;; After 256 frames, the enemy dies
    sys_ai_behaviour_enemy_no_v_shot:
    ld   a, enemy_counter(ix)
    cp   #0xFF
    jr   z, sys_ai_behaviour_h_enemy_destroy

    call sys_ai_behaviour_left_right
    ret

    sys_ai_behaviour_h_enemy_destroy:
    call man_entity_set4Destruction
    ret
;

;; Polymorphic function that determines the enemy behaviour
;; depending on the type of enemy and its funtion pointers
;; INPUT: enemy entity recieved in IX
sys_ai_behaviour_dashing_enemy::
    ;; Render sprite solid box
;;    ld   bc, #enemy_anguila_dead_template
;;    ld   a, #TYPE_ENEMY_SHOT
;;    call man_game_create_template_entity
;;    ld bc, #0x0000
;;    call man_entity_setRelativePos
;;    ld bc, #entity_prevvmemptr
;;    ex de,hl
;;    add hl,bc
;;    ld a, entity_prevvmemptr(ix)
;;    ld (hl),a
;;    inc hl
;;    ld a, entity_prevvmemptr+1(ix)
;;    ld (hl),a

    ld   a, entity_vy(ix)
    or   a
    jr   nz, sys_ai_behaviour_dashing_enemy_no_shot
    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #63
    jr   nz, sys_ai_behaviour_dashing_enemy_no_shot

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_dashing_enemy_return
    push bc
    jp (hl)
    sys_ai_behaviour_dashing_enemy_return:

    ;; After 256 frames, the enemy dies
    sys_ai_behaviour_dashing_enemy_no_shot:
    ld   a, enemy_counter(ix)
    cp   #0xFF
    jr   z, sys_ai_behaviour_dashing_enemy_destroy

    ld   a, (dash_state)
    or   a
    jr   z, dash_wait

    ld   a, enemy_counter(ix)
    and  #7
    ret  nz
    call sys_ai_behaviour_vertical_dashes
    ld   a, enemy_counter(ix)
    and  #31
    ret  nz
    ld   a, #0
    ld   (dash_state), a
    ret

    dash_wait:
    ld   entity_vy(ix), #0
    ld   a, enemy_counter(ix)
    and  #63
    ret  nz
    ld   a, #1
    ld   (dash_state), a
    ret

    sys_ai_behaviour_dashing_enemy_destroy:
    ld   a, enemy_aux(ix)
    inc  a
    ld   enemy_aux(ix), a
    cp   #1
    ret  z
    call man_entity_set4Destruction
    ret
;

;; Polymorphic function that determines the enemy behaviour
;; depending on the type of enemy and its funtion pointers
;; INPUT: enemy entity recieved in IX
sys_ai_behaviour_teleporting_enemy::
    ld   entity_vy(ix), #0
    ld   entity_vx(ix), #0

    ;; Every 16 frames, the enemy shots
    ld   a, enemy_counter(ix)
    and  #31
    jr   nz, sys_ai_behaviour_teleporting_enemy_no_shot

    ;; Loads the pointer to the shot_patter function
    ld l, enemy_shot_pattern(ix)
    ld h, 1 + enemy_shot_pattern(ix)
    ld bc, #sys_ai_behaviour_teleporting_enemy_return
    push bc
    jp (hl)
    sys_ai_behaviour_teleporting_enemy_return:

    ;; After 256 frames, the enemy dies
    sys_ai_behaviour_teleporting_enemy_no_shot:
    ld   a, enemy_counter(ix)
    ;;inc  a
    ;;ld   enemy_counter(ix), a
    cp   #0xFF
    jr   z, sys_ai_behaviour_teleporting_enemy_destroy

    ld   a, enemy_counter(ix)
    and  #63
    ret  nz
    call sys_ai_behaviour_teleport
    ret

    sys_ai_behaviour_teleporting_enemy_destroy:
    call man_entity_set4Destruction
    ret
;

sys_ai_behaviour_enemy_follow::
    call sys_ai_behaviour_follow

    ld l,enemy_shot_pattern(ix)
    ld h,enemy_shot_pattern+1(ix)
    jp (hl) ;No hace falta push de vuelta porque es el final de la función
;

sys_ai_behaviour_explosion::
    ld a,enemy_counter(ix)
    cp #37
    ret nz
    call man_entity_set4Destruction
    ret
;