;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module AiMovements

.include "man/entity.inc"
.include "sys/render.inc"
.include "man/entity_templates.inc"
.include "globals.inc"

.area _DATA

first_time_enemy_sector:
    .db 0
;

dash_state:
    .db 0
;

.area _CODE

sys_ai_movements_init::
    xor a
    ld  (first_time_enemy_sector), a
    ld  (dash_state), a
    ret
;

sys_ai_behaviour_regenerate::
    ld   a, #255
    ld   enemy_health(ix), a    ;; Para regenerarse
    ret
;

;Entity to update goes in IX
;Destroys A, B
sys_ai_behaviour_left_right:
    ld a, #RIGHT_BOUND
    sub entity_width(ix)
    ld b, a                ; b = right_bound - e_width
    ld a, entity_x(ix)
    or a                   ; if(a == 0)
    jr z, go_right
    cp b                   ; elseif(a == b)
    ret nz

    ;; Go left
    ld entity_vx(ix), #-1
    ret

    go_right:
    ld entity_vx(ix), #1
    ret
;

;Entity to update goes in IX
;Destroys A, B
sys_ai_behaviour_up_down::
    ld a, #BOTTOM_BOUND
    sub entity_height(ix)
    ld b, a                ; b = bottom_bound - e_height
    ld a, entity_y(ix)
    or a                   ; if(a == 0)
    jr z, go_down
    cp b                   ; elseif(a == b)
    ret nz

    ;; Go up
    ld entity_vy(ix), #-1
    ret

    go_down:
    ld entity_vy(ix), #1
    ret
;

sys_ai_behaviour_vertical_dashes:
    ;; We recover the direction of the enemy
    ld  a, (dash_state)
    ld  entity_vy(ix), a

    ld  a, entity_y(ix)
    add entity_vy(ix)
    ld  b, a                ; b = next position of enemy

    ld  a, entity_vy(ix)
    add a
    jr  nc, going_down

    going_up:
    ld  a, #50              ; a = highest point where enemy can reach
    cp  b
    jr  nc, turn_back

    dec entity_vy(ix)

    ;; We save the direction of the enemy
    ld  a, entity_vy(ix)
    ld  (dash_state), a
    ret

    going_down:
    ld  a, #BOTTOM_BOUND - 50
    sub entity_height(ix)   ; a = lowest point where enemy can reach
    cp  b
    jr  nc, accelerate_down

    ld  a, #-1
    ld  entity_vy(ix), a

    ;; We save the direction of the enemy
    ld  a, entity_vy(ix)
    ld  (dash_state), a
    ret

    turn_back:
    ld  a, #1
    ld  entity_vy(ix), a

    ;; We save the direction of the enemy
    ld  a, entity_vy(ix)
    ld  (dash_state), a
    ret

    accelerate_down:
    inc entity_vy(ix)
    
    ;; We save the direction of the enemy
    ld  a, entity_vy(ix)
    ld  (dash_state), a
    ret
;

sys_ai_behaviour_teleport::
    ld   bc, #enemy_depths_fish_dead_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ld bc, #0x0000
    call man_entity_setRelativePos
    ld bc, #entity_prevvmemptr
    ex de,hl
    add hl,bc
    ld a, entity_prevvmemptr(ix)
    ld (hl),a
    inc hl
    ld a, entity_prevvmemptr+1(ix)
    ld (hl),a

    ld  a, enemy_aux(ix)    ; teleport state
    ld  b, #2
    cp  b
    jr  nc, to_origin
    ld  b, #1
    cp  b
    jr  nc, to_front

    ;; To down
    ld  a, #100
    ld  entity_vy(ix), a
    ld  a, #1
    ld  enemy_aux(ix), a    ; teleport state
    ret

    to_front:
    ld  a, #-20
    ld  entity_vx(ix), a
    ld  a, #-50
    ld  entity_vy(ix), a
    ld  a, #2
    ld  enemy_aux(ix), a    ; teleport state
    ret

    to_origin:
    ld  a, #20
    ld  entity_vx(ix), a
    ld  a, #-50
    ld  entity_vy(ix), a
    ld  a, #0
    ld  enemy_aux(ix), a    ; teleport state
    ret
;

sys_ai_behaviour_circle::
    ;; Render sprite solid box
    ld   bc, #boss_squid_dead_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ld bc, #0x0000
    call man_entity_setRelativePos
    ld bc, #entity_prevvmemptr
    ex de,hl
    add hl,bc
    ld a, entity_prevvmemptr(ix)
    ld (hl),a
    inc hl
    ld a, entity_prevvmemptr+1(ix)
    ld (hl),a

    ;; Check if it is in sector 01 or 12
    ld  a, entity_x(ix)
    cp  #CIRCLE_CENTER_X
    jr  c, sector_01_12

    ;; Check if it is in sector 23 or 30
    ld  b, entity_y(ix)
    ld  a, #CIRCLE_CENTER_Y
    cp  b
    jr  c, sector_23

    ;; It is in sector 30
    ld  a, (first_time_enemy_sector)
    or  a
    jr  z, no_first_time_sector_30

    ;; If first time in this sector
    ld  a, #0
    ld  (first_time_enemy_sector), a
    ld  entity_vx(ix), #-1

    no_first_time_sector_30:
    inc entity_vy(ix)
    ret

    sector_23:
    ld  a, (first_time_enemy_sector)
    or  a
    jr  nz, no_first_time_sector_23

    ;; If first time in this sector
    ld  a, #1
    ld  (first_time_enemy_sector), a
    ld  entity_vy(ix), #-1
    ret

    no_first_time_sector_23:
    dec entity_vy(ix)
    ret

    sector_01_12:
    ;; Check if it is in sector 12
    ld  b, entity_y(ix)
    ld  a, #CIRCLE_CENTER_Y
    cp  b
    jr  c, sector_12

    ;; It is in sector 01
    ld  a, (first_time_enemy_sector)
    or  a
    jr  nz, no_first_time_sector_01

    ;; If first time in this sector
    ld  a, #1
    ld  (first_time_enemy_sector), a
    ld  entity_vy(ix), #1

    no_first_time_sector_01:
    inc entity_vy(ix)
    ret

    sector_12:
    ld  a, (first_time_enemy_sector)
    or  a
    jr  z, no_first_time_sector_12

    ;; If first time in this sector
    ld  a, #0
    ld  (first_time_enemy_sector), a
    ld  entity_vx(ix), #1

    no_first_time_sector_12:
    dec entity_vy(ix)
    ret
;

;Enemy to update goes in IX
sys_ai_behaviour_follow::
    ld a,enemy_counter(ix)
    and #7
    ret nz

    ld iy,#player
    ld a, entity_y(ix)
    cp entity_y(iy)
    ret z
    jp c, lower
    dec entity_vy(ix)
    ret
    lower:
    inc entity_vy(ix)
    ret
;