;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module AiShots

.include "man/entity.inc"
.include "sys/render.inc"
.include "man/entity_templates.inc"
.include "globals.inc"

.area _DATA
.area _CODE

;; Creates a single shot pattern
;; OUTPUT: entity shot created goes in HL
sys_ai_shot_pattern_single_shot::
    call man_game_enemy_shot
    ld bc,#0x02FE
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_pattern_tripleShot::
    call man_game_enemy_shot
    ld bc,#0x02FE
    call man_entity_setRelativePos

    call man_game_enemy_shot
    ld bc,#0xFE02
    call man_entity_setRelativePos

    call man_game_enemy_shot
    ld bc,#0x0602
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_pattern_shot_crab::
    call man_game_enemy_shot
    ld bc,#0x1EFE
    call man_entity_setRelativePos

    call man_game_enemy_shot
    ld bc,#0x1A02
    call man_entity_setRelativePos
    
    call man_game_enemy_shot
    ld bc,#0x2202
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_pattern_arco::
    call man_game_enemy_shot
    ld bc,#0xFD01
    call man_entity_setRelativePos

    call man_game_enemy_shot
    ld bc,#0x01FF
    call man_entity_setRelativePos

    call man_game_enemy_shot
    ld bc,#0x09FF
    call man_entity_setRelativePos

    call man_game_enemy_shot
    ld bc,#0x0D01
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_pattern_big_shot::
    ld   bc, #big_shot_enemy_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ld bc,#0x02FE
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_pattern_big_shot_mecha_kraken:
    ld   bc, #big_shot_enemy_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ld bc,#0x2AFE
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_pattern_mega_shot_mecha_kraken:
    ld   bc, #mega_shot_enemy_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ld bc,#0x1FFE
    call man_entity_setRelativePos
    ret
;


sys_ai_shot_pattern_vertical_shot::
    call man_game_enemy_vertical_shot
    ret
;

;; Shot pattern for squid boss (4 shots)
;; OUTPUT: the last of the 4 shots goes in HL
sys_ai_shot_pattern_boss_squid_shot::
    ld   hl, #_spr_boss_squid_shot
    ld   entity_spritePtr(ix), l
    ld   1 + entity_spritePtr(ix), h
    ld   a, #24
    ld   enemy_animCounter(ix), a
    call sys_ai_shot_pattern_arco
    ret
;

;; Shot pattern for crab boss, which switches
;; between 2 patterns depending on some conditions
;; OUTPUT:
sys_ai_shot_pattern_boss_crab_shot::
    ld   a, entity_vy(ix)
    or   a
    jr   z, crab_bombazo
    call sys_ai_shot_pattern_shot_crab
    ret

    crab_bombazo:
    call sys_ai_shot_pattern_big_shot
    ld   hl, #_spr_boss_crab_shot
    ld   entity_spritePtr(ix), l
    ld   1 + entity_spritePtr(ix), h
    ld   a, #12
    ld   enemy_animCounter(ix), a
    ret
;

sys_ai_shot_pattern_boss_mechaKraken_shot_2::
    ld  a, entity_y(ix)
    cp  #42
    ret nz

    call sys_ai_shot_pattern_mega_shot_mecha_kraken
    ld   hl, #_spr_mecha_kraken_shot
    ld   entity_spritePtr(ix), l
    ld   1 + entity_spritePtr(ix), h
    ld   a, #12
    ld   enemy_animCounter(ix), a
    ret
;

sys_ai_shot_pattern_boss_mechaKraken_shot::
    ld  a, enemy_counter(ix)
    and #31
    jr  z, mechaKraken_hadoken
    ret

    mechaKraken_hadoken:
    call sys_ai_shot_pattern_big_shot_mecha_kraken
    ret
;

sys_ai_shot_big_dead::
    ld   bc, #shot_follow_template
    ld   a, #TYPE_ENEMY_SHOT
    call man_game_create_template_entity
    ld bc,#0x0000
    call man_entity_setRelativePos
    ld bc, #entity_prevvmemptr
    ex de,hl
    add hl,bc
    ld a, entity_prevvmemptr(ix)
    ld (hl),a
    inc hl
    ld a, entity_prevvmemptr+1(ix)
    ld (hl),a
    call man_entity_setRelativePos
    ret
;

sys_ai_shot_empty::
    ret
;

sys_ai_shot_enemy::
    ld bc, #enemy_follow_template
    ld a, #TYPE_ENEMY
    call man_game_create_template_entity
    ld bc, #0x0002
    call man_entity_setRelativePos
    ret
;