;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module Collision

.include "globals.inc"
.include "man/entity.inc"
.include "man/game.inc"
.include "cpctelera.h.s"

.area _DATA

damage_effect:
    .db 0
;    
damage_cooldown:
    .db COOLDOWN_TIME
;
COOLDOWN_TIME   = 50
HIT_DAMAGE      = 16
.area _CODE

HALF_SCREEN = 40 

;Input - IY: First entity
;Input - IX: Second entity
;TODO Se puede optimizar quitando las velocidades, si no se puede dar el caso de que se crucen
sys_collisions_check:
    ld a,entity_x(ix)
    cp entity_x(iy)            ;x+W > x
    ret nc

    add entity_width(ix)
    ld c,a
    ld a, entity_x(iy)            ;x-vx < x+W-vx
    cp c
    ret nc


    
    ld a,entity_y(ix)
    cp entity_y(iy)            ;x+W > x
    ret nc

    add entity_height(ix)
    ld c,a
    ld a, entity_y(iy)            ;x-vx < x+W-vx
    cp c
    ret
    ;Acabar aquí
;
sys_collisions_check_bounding_box:
    ld a,entity_x(iy)
    add entity_width(iy)
    add c
    ld c,a
    ld a,entity_x(ix)
    cp c            ;x+W > x
    ret nc

    add entity_width(ix)
    ld c,a
    ld a,entity_x(iy)
    add b
    cp c            ;x-vx < x+W-vx
    ret nc


    ld a,entity_y(iy)
    add entity_height(iy)
    add e
    ld c,a
    ld a,entity_y(ix)
    cp c            ;y+H > y
    ret nc

    add entity_height(ix)
    ld c,a
    ld a,entity_y(iy)
    add d
    cp c            ;y-vy < y+H-vy
    ret
    ;Acabar aquí
;

sys_collisions_shot_enemy:
    call sys_collisions_check
    ret nc

    dec enemy_health(ix)
    xor a
    cp enemy_health(ix)
    jr nz, not_dead

    ld   l, enemy_collisionPtr(ix)
    ld   h, 1 + enemy_collisionPtr(ix)
    ld   a, h
    or   a
    jr   z, not_mecha_kraken
    push ix
    push iy
    ld   bc, #mecha_kraken_dead_return
    push bc
    jp   (hl)
    mecha_kraken_dead_return:
    pop  iy
    pop  ix

    not_mecha_kraken:
    call man_entity_set4Destruction
    call update_score

    ld bc,#enemy_explosion_template
    ld a,#TYPE_ENEMY
    call man_game_create_template_entity
    ex de,hl
    ld bc,#entity_x
    add hl,bc
    ld a, entity_x(ix)
    ld (hl),a
    inc hl
    ld a, entity_y(ix)
    ld (hl),a
    not_dead:
    push ix
    push iy
    pop ix
    call man_entity_set4Destruction
    pop ix

    ret
;

;Input - IX: First entity
sys_collisions_first_type:
    ld a,entity_x(ix)
    cp #HALF_SCREEN
    ret c
    push ix
    pop iy

    ld a,#TYPE_ENEMY
    ld hl, #sys_collisions_shot_enemy
    call man_entity_forAll_matching

    push iy
    pop ix

    ret
;

sys_collisions_shot_player:
    ld bc,#0x03FE
    ld de,#0x06FD
    call sys_collisions_check_bounding_box
    ret nc
    
    call man_entity_set4Destruction

    call sys_collisions_damage

    ret
;

sys_collisions_enemy_player:
    ld bc,#0x03FE
    ld de,#0x06FD
    call sys_collisions_check_bounding_box
    ret nc

    call sys_collisions_damage_enemy

    ret
;
sys_collisions_update::
    ld  a, (damage_effect)
    or  a
    jr  z, init_collisions_update

        cp #1
        jr z, level_palette
            dec a
            ld (damage_effect), a
            jr init_collisions_update
        level_palette:
            dec a
            ld (damage_effect), a
            ld a,(current_level)
            call sys_render_set_level_palette

    init_collisions_update:
    ld      hl, #sys_collisions_first_type
    ld      a, #TYPE_PLAYER_SHOT
    call    man_entity_forAll_matching


    ld iy,#player
    ld      hl, #sys_collisions_shot_player
    ld      a, #TYPE_ENEMY_SHOT
    call    man_entity_forAll_matching

    ld iy,#player
    ld      hl, #sys_collisions_enemy_player
    ld      a, #TYPE_ENEMY
    call    man_entity_forAll_matching

    ld a,(damage_cooldown)
    cp #COOLDOWN_TIME
    ret z
    inc a
    ld (damage_cooldown),a

    and #1
    ret z

    ld hl, #_spr_vacio
    ld ix, #player
    ld entity_spritePtr(ix),l
    ld entity_spritePtr+1(ix),h

    ret
;

sys_collisions_damage:
    ld a,(damage_cooldown)
    cp #COOLDOWN_TIME
    ret c

    xor a
    ld (damage_cooldown),a

    ld a,player_health(iy)
    add shot_damage(ix)
    ld player_health(iy),a

    push af
    call sys_render_damage_palette
    ld a, #4
    ld (damage_effect), a
    pop af

    ret c
    ld a,#STATE_GAME_OVER
    call man_game_setGameState

    ret
;
sys_collisions_damage_enemy:
    ld a,(damage_cooldown)
    cp #COOLDOWN_TIME
    ret c

    xor a
    ld (damage_cooldown),a

    ld a,player_health(iy)
    add #-16
    ld player_health(iy),a

    push af
    call sys_render_damage_palette
    ld a, #4
    ld (damage_effect), a
    pop af

    ret c
    ld a,#STATE_GAME_OVER
    call man_game_setGameState

    ret
;
update_score:
    push af
    push bc
    push de
    push hl
    push ix
    push iy
    ld b,#5
    ld hl, #score+#4
    call uscor
    ld hl,#0x010f
    call cpct_setDrawCharM0_asm
    ld de,#CPCT_VMEM_START_ASM
    ld b,#192
    ld c,#50
    call cpct_getScreenPtr_asm
    ld iy,#score
    call cpct_drawStringM0_asm
    pop iy
    pop ix
    pop hl
    pop de
    pop bc
    pop af
    ret
;