;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module Input

.include "sys/input.inc"
.include "man/entity.inc"
.include "sys/render.inc"
.include "man/entity_templates.inc"
.include "globals.inc"
.include "cpctelera.h.s"

.area _DATA 

input_option:
  .db 0
;

pulse_effect:
  .db 0
;

keyboard_keys:
  .dw Key_O
  .dw Key_P
  .dw Key_Q
  .dw Key_A
  .dw Key_Space
;
joystick_keys:
  .dw Joy0_Left
  .dw Joy0_Right
  .dw Joy0_Up
  .dw Joy0_Down
  .dw Joy0_Fire1
;

PULSE_COOLDOWN = 128  ;TODO: no cooldown, solo soltar y pulsar
PULSE_COST  = 8
last_frame_pulse:
  .db 0
;

.area _CODE

man_game_getInputOption::
    ld a,(input_option)
    ret
;
man_game_setInputOption::
    ld (input_option),a
    ret
;

;Entity to update goes in IX
;Destroys AF,BC,DE,HL
sys_input_checkInputs:
    ld   hl, #_spr_playership
    ld   entity_spritePtr(ix), l
    ld   1 + entity_spritePtr(ix), h
    ;;call cpct_scanKeyboard_asm
    ld e, #0
    
    ld a, (input_option)
    or a
    jr z, keyboard_option_selected
      ld iy, #joystick_keys
      jr start_checking

    keyboard_option_selected:
      ld iy, #keyboard_keys
    start_checking:
    ;Left Right movement
     ;test_key_O
      ld l, action_move_left_0(iy)
      ld h, action_move_left_1(iy)
      call cpct_isKeyPressed_asm
      jr z, test_key_P
        ld a, entity_x(ix)
        or a
        jr z, test_key_P
      dec e
     test_key_P:
      ld l, action_move_right_0(iy)
      ld h, action_move_right_1(iy)
      call cpct_isKeyPressed_asm
      jr z, set_speed_vx
        ld a, #RIGHT_BOUND
        sub entity_width(ix)
        ld b, a
        ld a, entity_x(ix)
        cp b
        jr z, set_speed_vx
      inc e
     set_speed_vx:
      ld entity_vx(ix),e
    ;

    ;Up Down movement
      ld e, #0
      ld l, action_move_up_0(iy)
      ld h, action_move_up_1(iy)
      call cpct_isKeyPressed_asm
      jr z, test_key_A
        ld a, entity_y(ix)
        or a
        jr z, test_key_A
      dec e
      dec e
     test_key_A:
      ld l, action_move_down_0(iy)
      ld h, action_move_down_1(iy)
      call cpct_isKeyPressed_asm
      jr z, set_speed_vy
        ld a, #BOTTOM_BOUND
        sub entity_height(ix)
        ld b, a
        ld a, entity_y(ix)
        cp b
        jr z, set_speed_vy
      inc e
      inc e
     set_speed_vy:
      ld entity_vy(ix),e
      ld a, entity_vy(ix)
      add a
      jr  c, going_up_sprite

      ld   a, entity_vy(ix)
      or   a
      jr   z, continue_shot

      ;; Going down sprite
      ld   hl, #_spr_playership_down
      ld   entity_spritePtr(ix), l
      ld   1 + entity_spritePtr(ix), h
      jp   continue_shot

      going_up_sprite:
      ld   hl, #_spr_playership_up
      ld   entity_spritePtr(ix), l
      ld   1 + entity_spritePtr(ix), h
    ;

    continue_shot:
     inc player_timeLastShot(ix)
     ld a,#0x07
     and player_timeLastShot(ix)
     jr nz, too_few_frames
     ld player_lastFrameSpace(ix),#0
     too_few_frames:
     xor a
     cp player_lastFrameSpace(ix)
     jr nz, space_not_pressed
     ld l, action_shot_0(iy)
     ld h, action_shot_1(iy)
     call cpct_isKeyPressed_asm
     jr z, space_not_pressed
     ld player_lastFrameSpace(ix),#1
     call man_game_player_shot
     ld player_timeLastShot(ix),#0
     ret
    space_not_pressed:
    ;Regeneración
     ld a,#0x0F      ;TODO constante de regeneración
     and player_timeLastShot(ix)
     ret nz
     ld a, player_health(ix)
     cp #PlayerShip_0_INIT_HEALTH
     ret z
     inc player_health(ix)  
    ret
;

sys_input_checkPlayer::
  call man_game_getInputOption
  call sys_input_checkInputs

    ld a, (pulse_effect)
    or a
    jr z, check_shift
      dec a
      ld (pulse_effect), a
      call sys_render_pulse_palette

  check_shift:
  ld hl,#Key_Shift
  call cpct_isKeyPressed_asm
  jp z, no_pulse

  ld hl,#last_frame_pulse
  xor a
  cp (hl)
  ret nz
  ld (hl),#1
  
  ld a,player_health(ix)
  cp #PULSE_COST
  jp c, no_pulse
  add #-PULSE_COST
  ld player_health(ix),a
  ld hl,#man_entity_set4Destruction
  ld a,#TYPE_ENEMY_SHOT
  push ix
  call man_entity_forAll_matching
  pop ix
    ld a, #9
    ld (pulse_effect), a
    call sys_render_pulse_palette
  ret

  no_pulse:
  ld hl,#last_frame_pulse
  ld (hl),#0
  ret
;








