;MIT License
;
;Copyright (c) 2022 cabbage_corp / Alberto Rius Poveda (@ariusp) / Alfonso Ruiz MartÃ­nez (@NexiriusRM) / Eliseo Fuentes MartÃ­nez (@efm18)
;
;Permission is hereby granted, free of charge, to any person obtaining a copy
;of this software and associated documentation files (the "Software"), to deal
;in the Software without restriction, including without limitation the rights
;to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;copies of the Software, and to permit persons to whom the Software is
;furnished to do so, subject to the following conditions:
;
;The above copyright notice and this permission notice shall be included in all
;copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;SOFTWARE.

.module Render

.include "cpctelera.h.s"
.include "man/entity.inc"
.include "sys/render.inc"
.include "man/entity_templates.inc"
.include "globals.inc"

.area _DATA
diff_hp_player_bar:
    .db 0
;

.area _CODE


sys_render_one_entity:
    ;Dont draw if dead
     ld a,entity_type(ix)
     and #TYPE_DEAD
     cp #TYPE_DEAD
     jr z, erase_sprite
    ;Get position to draw
     ld de,#CPCT_VMEM_START_ASM
     ld c,entity_x(ix)
     ld b,entity_y(ix)
     call cpct_getScreenPtr_asm  ;HL holds screen pointer
    ;
     
     ex de, hl
     ld entity_prevvmemptr(ix),e
     ld entity_prevvmemptr+1(ix),d  ;Store pointer to memory

     ld l, entity_spritePtr(ix)
     ld h, 1+entity_spritePtr(ix)
     ld c, entity_width(ix)
     ld b, entity_height(ix)
     call cpct_drawSprite_asm

     ld a, entity_type(ix)
     cp #TYPE_PLAYER
     ret nz
     call sys_render_update_hp_bar
     ret

    erase_sprite:
    ;Erase
     ld a,entity_prevvmemptr+1(ix)
     cp #0
     ret z
     ld  d, a
     ld e,entity_prevvmemptr(ix)
     xor a
     ld c,entity_width(ix)
     ld b,entity_height(ix)
     call cpct_drawSolidBox_asm  ;Erase sprite in previous position
    ret
;

sys_render_update_hp_bar:
    call man_game_getGameState
    cp #0
    jr nz, update_that_bar
    ld c,#HP_BAR_INIT_X
    ld b,#HP_BAR_INIT_Y
    ld de,#CPCT_VMEM_START_ASM
    call cpct_getScreenPtr_asm
    ex de,hl
    ld a,#DANYO
    ld c, #HP_BAR_W
    ld b, #HP_BAR_H
    call cpct_drawSolidBox_asm
    ret

    update_that_bar:
    ld  a, (diff_hp_player_bar)
    ld  b, player_health(ix)
    sub b 
    jp z, end_hp_update
    
        ld  a, (diff_hp_player_bar)
        sub b
        jr  c, hp_regeneration
            ;Player has taken damage
            ld  de, #CPCT_VMEM_START_ASM
            ld  c, b
            or  c

            jr  nz, damage_hp_more_than_0
                ld  c, #HP_BAR_INIT_X
                jr  damage_end_if_hp_equal_0

            damage_hp_more_than_0:
                rr  c
                ld  a, #HP_BAR_INIT_X
                add a, c
                ld  c, a

            damage_end_if_hp_equal_0:
            ld  b, #HP_BAR_INIT_Y
            call cpct_getScreenPtr_asm

            ex  de, hl
            ld  a, (diff_hp_player_bar)
            ld  b, player_health(ix)
            sub b
            rra
            jr nc, hp_danyo_pair
                ld  c, a
                inc c
                ld  a, #DANYO
                ld  b, #HP_BAR_H
                call cpct_drawSolidBox_asm
                jp   continue_danyo

                hp_danyo_pair:
                ld  c, a
                ld  a, #DANYO
                ld  b, #HP_BAR_H
                call cpct_drawSolidBox_asm

            continue_danyo:
            ld a, player_health(ix)
            ld (diff_hp_player_bar), a
            jr end_hp_update

        hp_regeneration:
            ;Player has healed
            ld  de, #CPCT_VMEM_START_ASM
            ld  a, (diff_hp_player_bar)
            or  a

            jr  nz, regeneration_hp_more_than_0
                ld  a, #HP_BAR_INIT_X
                jr  regeneration_end_if_hp_equal_0

            regeneration_hp_more_than_0:
                rra
                add  a, #HP_BAR_INIT_X

            regeneration_end_if_hp_equal_0:
            ld  c, a
            ld  b, #HP_BAR_INIT_Y
            call cpct_getScreenPtr_asm

            ex  de, hl
            ld  a, (diff_hp_player_bar)
            ld  b, a
            ld  a, player_health(ix)
            sub b
            rra

            jr c, end_hp_update
                ld  c, a
                ld  a, #VIDA
                ld  b, #HP_BAR_H
                call cpct_drawSolidBox_asm
                ld a, player_health(ix)
                ld (diff_hp_player_bar), a

    end_hp_update:
    ret
;

sys_render_update::
    ld hl, #sys_render_one_entity
    call man_entity_forAll

    ret
;

sys_render_init::

    ld de, #CPCT_VMEM_START_ASM
    ld bc,#BOTTOM_BOUND << 8
    call cpct_getScreenPtr_asm
    ex de,hl
    ld a, #0xC0
    ld bc, #0x1440
    call cpct_drawSolidBox_asm

    ld de, #CPCT_VMEM_START_ASM
    ld b,#BOTTOM_BOUND
    ld c,#64
    call cpct_getScreenPtr_asm
    ex de,hl
    ld a,#0xC0
    ld bc,#0x1410
    call cpct_drawSolidBox_asm

    ld de, #CPCT_VMEM_START_ASM
    ld c, #HP_BAR_INIT_X
    ld b, #HP_BAR_INIT_Y
    call cpct_getScreenPtr_asm

    ex de, hl
    ld a, #VIDA
    ld c, #HP_BAR_W
    ld b, #HP_BAR_H
    call cpct_drawSolidBox_asm

    ld  a, #PlayerShip_0_INIT_HEALTH
    ld  (diff_hp_player_bar), a


    ld de, #CPCT_VMEM_START_ASM
    ld bc,#BOTTOM_BOUND << 8
    call cpct_getScreenPtr_asm
    ex de,hl
    ld b, #10
    ld c, #20
    ld hl, #_spr_rocks
    call cpct_drawSprite_asm

    ld de, #CPCT_VMEM_START_ASM
    ld b, #BOTTOM_BOUND
    ld c, #20
    call cpct_getScreenPtr_asm
    ex de,hl
    ld b, #10
    ld c, #20
    ld hl, #_spr_rocks
    call cpct_drawSprite_asm

    ld de, #CPCT_VMEM_START_ASM
    ld b, #BOTTOM_BOUND
    ld c, #40
    call cpct_getScreenPtr_asm
    ex de,hl
    ld b, #10
    ld c, #20
    ld hl, #_spr_rocks
    call cpct_drawSprite_asm

    ld de, #CPCT_VMEM_START_ASM
    ld b, #BOTTOM_BOUND
    ld c, #60
    call cpct_getScreenPtr_asm
    ex de,hl
    ld b, #10
    ld c, #20
    ld hl, #_spr_rocks
    call cpct_drawSprite_asm

    ld hl,#0x010f
    call cpct_setDrawCharM0_asm
    ld de,#CPCT_VMEM_START_ASM
    ld b,#192
    ld c,#50
    call cpct_getScreenPtr_asm
    ld iy,#score
    call cpct_drawStringM0_asm
    ret 
;

;Input  - A:  Level number
;Output - HL: Palette
sys_render_set_level_palette::
    ld de,#PALETTE_SIZE
    dec a
    jr nz, level_2
    ld hl,#_palette_level_1
    jp set_palette
    level_2:
    dec a
    jr nz, level_3
    ld hl,#_palette_level_2
    jp set_palette
    level_3:
    ld hl,#_palette_level_3
    set_palette:
    call cpct_setPalette_asm
    ret
;

sys_render_pulse_palette::
    or a
    jr nz, check_anim_1
    ld a,(current_level)
    call sys_render_set_level_palette
    jr end_change_palette


    check_anim_1:
    cp #6
    jr nc, check_anim_2
    cp #3
    jr c, check_anim_2
    ld hl,#_palette_pulse_2
    ld de,#PALETTE_SIZE
    call cpct_setPalette_asm
    jr end_change_palette

    check_anim_2:
    ld hl,#_palette_pulse_1
    ld de,#PALETTE_SIZE
    call cpct_setPalette_asm

    end_change_palette:
    ret
;

sys_render_damage_palette::
    ld hl,#_palette_damage
    ld de,#PALETTE_SIZE
    call cpct_setPalette_asm

    
    ret
;

;Input  - A: Number of level
;Destroys AF,BC,DE,HL
;sys_render_set_level_palette::
;    cp  a, #3
;    jr  nz, check_if_level_2
;        ld hl,#_palette_pulse_2
;        ld de,#PALETTE_SIZE
;        call cpct_setPalette_asm
;        jr end_set_level_palette
;
;    check_if_level_2:
;    cp  a, #2
;    jr  nz, check_if_level_1
;        ld hl,#_palette_pulse_2
;        ld de,#PALETTE_SIZE
;        call cpct_setPalette_asm
;        jr end_set_level_palette
;
;    check_if_level_1:
;        ld hl,#_palette_pulse_2
;        ld de,#PALETTE_SIZE
;        call cpct_setPalette_asm
;
;    end_set_level_palette:
;    ret
;